////////////////////////////////////////////////////////////////////////
// Matballz.fx
//
//	Shader by Charles Hollemeersch http://charles.hollemeersch.net/
//	Edited: 12/13/2009 - Leonardo Covarrubias - For use in Maya
//	Public domain.
//	Edited 15/06/2015 - Royal Ghost - For use in 3ds max Nitrous directx 11.
//
//	Parameters:
//
//	  Bumpmap Strength: Scales the bumps to be flatter/steeper
//                          (i.e. more/less towards 0,0,1).
//	  Global Opacity  : Sets the uniform alpha value for the material.
//	  Normal Map      : Provides a bumpmap. If no map is provided,
//			    the geometry normal is used.
//	  Lit Sphere Map  : The prelighted sphere to apply as a light. Check
//                          http://www.cs.utah.edu/npr/papers/LitSphere_HTML/
//                          to get some inspiration.
//
////////////////////////////////////////////////////////////////////////

string ParamID = "0x003";

////////////////////////
// Matrices
////////////////////////

float4x4 WorldView            : WorldView <string UIWidget="None";>;
float4x4 WorldViewProjection  : WorldViewProjection < string UIWidget="None"; >;

////////////////////////
// User interface
////////////////////////

float BumpHeight <
	string UIName = "Bumpmap Strength";
	string UIWidget = "slider";
	float UIMin = 0.0f;
	float UIMax = 2.0f;
	float UIStep = 0.1f;
> = 1.0f;

float Alpha <
	string UIName = "Global Opacity";
	string UIWidget = "slider";
	float UIMin = 0.0f;
	float UIMax = 1.0f;
	float UIStep = 0.05f;
> = 1.0f;

bool flipGreen
<
    string UIName = "Invert Normal Map Green Channel?";
> = false;

Texture2D <float4> normalMap <
	string UIName = "Normal Map";
	string ResourceName = "";
	string ResourceType = "2D";
>;

Texture2D <float4> litSphereMap <
	string UIName = "LitSphere Map";
	string ResourceName = "./matball01.jpg";
	string ResourceType = "2D";
>;

////////////////////////
// Samplers
////////////////////////

//sampler NormalSampler = sampler_state {
SamplerState NormalSampler {

//	Texture = <normalMap>;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
    AddressU  = WRAP;		
    AddressV  = WRAP;	
};

//sampler LightSampler = sampler_state {
SamplerState LightSampler {
//	Texture = <litSphereMap>;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
    AddressU  = CLAMP;		
    AddressV  = CLAMP;
};

////////////////////////
// Exchange structs
////////////////////////

struct a2v 
{
	float4 Position : POSITION; 
	float2 TexCoord : TEXCOORD0;
	float3 Tangent  : TANGENT;
	float3 Binormal : BINORMAL;
	float3 Normal   : NORMAL;
};

struct v2f 
{
//	float4 Position : POSITION;
	float4 Position : SV_Position;
	float2 TexCoord : TEXCOORD0;
	float3 TexCoord1 : TEXCOORD1;
	float3 TexCoord2 : TEXCOORD2;
	float3 TexCoord3 : TEXCOORD3;
};

////////////////////////
// Vertex shader
////////////////////////

v2f BumpReflectVS(a2v IN)
{
	v2f OUT;

	// Pos to NDC
	OUT.Position = mul(float4(IN.Position.xyz,1), WorldViewProjection);
	
	// Texcoords (for normal map)
	OUT.TexCoord.xy = IN.TexCoord;
	
	// Tangent space vectors
	float3 vtan = IN.Tangent;
	float3 vbinorm = IN.Binormal;
	float3 vnorm = IN.Normal;

	OUT.TexCoord1.xyz = mul(vtan,WorldView);
	OUT.TexCoord2.xyz = mul(vbinorm,WorldView);
	OUT.TexCoord3.xyz = mul(vnorm,WorldView);
	
	return OUT;
}

////////////////////////
// Pixel shader
////////////////////////

//float4 BumpReflectPS(v2f IN) : COLOR {
float4 BumpReflectPS(v2f IN) : SV_Target {
//	float3 texNormal = tex2D(NormalSampler, IN.TexCoord.xy).xyz;
	float3 texNormal = normalMap.Sample(NormalSampler, IN.TexCoord.xy).xyz;
	if(flipGreen)texNormal.g = 1-texNormal.g;
	texNormal.rgb = texNormal.rgb * 2.0 - 1.0;
	// Fixes normals if no normal map texture is supplied
	if ( dot(texNormal,texNormal) > 2.0 ) {
		texNormal = float3(0.0,0.0,1.0);
	}
	
	// The normalizes can probably go away... but meh...
    float3 T = normalize(IN.TexCoord1.xyz) * BumpHeight;
    float3 B = normalize(IN.TexCoord2.xyz) * BumpHeight;
    float3 N = normalize(IN.TexCoord3.xyz);
		
	// Put in world space and renormalize (after scaling)
    float3 worldNorm = normalize(N + texNormal.y * T + texNormal.x * B);

	// Swap it around a bit... 
	worldNorm.y = - worldNorm.y;

	// Look up in the litsphere!
//	float4 light = tex2D(LightSampler, worldNorm.xy * 0.5 + 0.5);
	float4 light = litSphereMap.Sample(LightSampler, worldNorm.xy * 0.5 + 0.5);
	light.w *= Alpha; 
	return light;
}

////////////////////////
// Pixel shader
////////////////////////
technique11 Matballz < string Script = "Pass=p0;"; > /*not sure what these annotations do just copy pasted them from somewhere... */
{
	pass p0 < string Script = "Draw=geometry;"; > {		
//		VertexShader = compile vs_2_0 BumpReflectVS();
//		PixelShader = compile ps_2_0 BumpReflectPS();

	        SetVertexShader(CompileShader(vs_5_0,BumpReflectVS()));
//	        SetGeometryShader( NULL );
		SetPixelShader(CompileShader(ps_5_0,BumpReflectPS()));

	}
}
